body{
	background: white;
	color: black;
	font-family: sans-serif;
}
table{
	width: 100%;
	margin: auto;
	font-size: 13px;
}
table, th, td{
	border: 1px solid #d7d4d4;
	border-collapse: separate; /* Don't collapse */
  border-spacing: 0;
}
th{
	padding: 5px;
}
td{
	padding: 5px;
	min-width: 40px;
}
tr:nth-child(even) {background-color: #fffae6;}
thead{
  position: sticky;
  top: 0;
  background-color: white;
}
.tabContainer{
	padding-top:10px;
}
.button {
  background-color: #D4E6F1; /* Blue Gray */
  border: none;
  color: black;
  padding: 6px 3px;
  font-family: sans-serif;
  text-align: center;
  text-decoration: none;
  display: inline-block;
  font-size: 11px;
  transition-duration: 0.2s;
  position: relative;
  display: inline-block;
}
.button:hover {
  background-color: #d7d4d4;
  color: black;
}
.title{
    font-family: sans-serif;
    color: #ECF0F1;
    text-align: top;
}
progress {
  position: absolute;
  left: 50%;
  top: 50%;
}
input {
  /*border: 1px solid transparent;*/
  height: 25px;
  width: 500px;
  border: solid 1px black;
  border-radius: 3px;
  padding-left: 5px;
  cursor: pointer;
}
iframe {
  height: 90%;
  width: 100%;
}
